# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .status import (
    StatusResource,
    AsyncStatusResource,
    StatusResourceWithRawResponse,
    AsyncStatusResourceWithRawResponse,
    StatusResourceWithStreamingResponse,
    AsyncStatusResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from .prefixes.prefixes import (
    PrefixesResource,
    AsyncPrefixesResource,
    PrefixesResourceWithRawResponse,
    AsyncPrefixesResourceWithRawResponse,
    PrefixesResourceWithStreamingResponse,
    AsyncPrefixesResourceWithStreamingResponse,
)
from .allowlist.allowlist import (
    AllowlistResource,
    AsyncAllowlistResource,
    AllowlistResourceWithRawResponse,
    AsyncAllowlistResourceWithRawResponse,
    AllowlistResourceWithStreamingResponse,
    AsyncAllowlistResourceWithStreamingResponse,
)
from .syn_protection.syn_protection import (
    SynProtectionResource,
    AsyncSynProtectionResource,
    SynProtectionResourceWithRawResponse,
    AsyncSynProtectionResourceWithRawResponse,
    SynProtectionResourceWithStreamingResponse,
    AsyncSynProtectionResourceWithStreamingResponse,
)
from .tcp_flow_protection.tcp_flow_protection import (
    TCPFlowProtectionResource,
    AsyncTCPFlowProtectionResource,
    TCPFlowProtectionResourceWithRawResponse,
    AsyncTCPFlowProtectionResourceWithRawResponse,
    TCPFlowProtectionResourceWithStreamingResponse,
    AsyncTCPFlowProtectionResourceWithStreamingResponse,
)

__all__ = ["AdvancedTCPProtectionResource", "AsyncAdvancedTCPProtectionResource"]


class AdvancedTCPProtectionResource(SyncAPIResource):
    @cached_property
    def allowlist(self) -> AllowlistResource:
        return AllowlistResource(self._client)

    @cached_property
    def prefixes(self) -> PrefixesResource:
        return PrefixesResource(self._client)

    @cached_property
    def syn_protection(self) -> SynProtectionResource:
        return SynProtectionResource(self._client)

    @cached_property
    def tcp_flow_protection(self) -> TCPFlowProtectionResource:
        return TCPFlowProtectionResource(self._client)

    @cached_property
    def status(self) -> StatusResource:
        return StatusResource(self._client)

    @cached_property
    def with_raw_response(self) -> AdvancedTCPProtectionResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AdvancedTCPProtectionResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AdvancedTCPProtectionResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AdvancedTCPProtectionResourceWithStreamingResponse(self)


class AsyncAdvancedTCPProtectionResource(AsyncAPIResource):
    @cached_property
    def allowlist(self) -> AsyncAllowlistResource:
        return AsyncAllowlistResource(self._client)

    @cached_property
    def prefixes(self) -> AsyncPrefixesResource:
        return AsyncPrefixesResource(self._client)

    @cached_property
    def syn_protection(self) -> AsyncSynProtectionResource:
        return AsyncSynProtectionResource(self._client)

    @cached_property
    def tcp_flow_protection(self) -> AsyncTCPFlowProtectionResource:
        return AsyncTCPFlowProtectionResource(self._client)

    @cached_property
    def status(self) -> AsyncStatusResource:
        return AsyncStatusResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAdvancedTCPProtectionResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAdvancedTCPProtectionResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAdvancedTCPProtectionResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAdvancedTCPProtectionResourceWithStreamingResponse(self)


class AdvancedTCPProtectionResourceWithRawResponse:
    def __init__(self, advanced_tcp_protection: AdvancedTCPProtectionResource) -> None:
        self._advanced_tcp_protection = advanced_tcp_protection

    @cached_property
    def allowlist(self) -> AllowlistResourceWithRawResponse:
        return AllowlistResourceWithRawResponse(self._advanced_tcp_protection.allowlist)

    @cached_property
    def prefixes(self) -> PrefixesResourceWithRawResponse:
        return PrefixesResourceWithRawResponse(self._advanced_tcp_protection.prefixes)

    @cached_property
    def syn_protection(self) -> SynProtectionResourceWithRawResponse:
        return SynProtectionResourceWithRawResponse(self._advanced_tcp_protection.syn_protection)

    @cached_property
    def tcp_flow_protection(self) -> TCPFlowProtectionResourceWithRawResponse:
        return TCPFlowProtectionResourceWithRawResponse(self._advanced_tcp_protection.tcp_flow_protection)

    @cached_property
    def status(self) -> StatusResourceWithRawResponse:
        return StatusResourceWithRawResponse(self._advanced_tcp_protection.status)


class AsyncAdvancedTCPProtectionResourceWithRawResponse:
    def __init__(self, advanced_tcp_protection: AsyncAdvancedTCPProtectionResource) -> None:
        self._advanced_tcp_protection = advanced_tcp_protection

    @cached_property
    def allowlist(self) -> AsyncAllowlistResourceWithRawResponse:
        return AsyncAllowlistResourceWithRawResponse(self._advanced_tcp_protection.allowlist)

    @cached_property
    def prefixes(self) -> AsyncPrefixesResourceWithRawResponse:
        return AsyncPrefixesResourceWithRawResponse(self._advanced_tcp_protection.prefixes)

    @cached_property
    def syn_protection(self) -> AsyncSynProtectionResourceWithRawResponse:
        return AsyncSynProtectionResourceWithRawResponse(self._advanced_tcp_protection.syn_protection)

    @cached_property
    def tcp_flow_protection(self) -> AsyncTCPFlowProtectionResourceWithRawResponse:
        return AsyncTCPFlowProtectionResourceWithRawResponse(self._advanced_tcp_protection.tcp_flow_protection)

    @cached_property
    def status(self) -> AsyncStatusResourceWithRawResponse:
        return AsyncStatusResourceWithRawResponse(self._advanced_tcp_protection.status)


class AdvancedTCPProtectionResourceWithStreamingResponse:
    def __init__(self, advanced_tcp_protection: AdvancedTCPProtectionResource) -> None:
        self._advanced_tcp_protection = advanced_tcp_protection

    @cached_property
    def allowlist(self) -> AllowlistResourceWithStreamingResponse:
        return AllowlistResourceWithStreamingResponse(self._advanced_tcp_protection.allowlist)

    @cached_property
    def prefixes(self) -> PrefixesResourceWithStreamingResponse:
        return PrefixesResourceWithStreamingResponse(self._advanced_tcp_protection.prefixes)

    @cached_property
    def syn_protection(self) -> SynProtectionResourceWithStreamingResponse:
        return SynProtectionResourceWithStreamingResponse(self._advanced_tcp_protection.syn_protection)

    @cached_property
    def tcp_flow_protection(self) -> TCPFlowProtectionResourceWithStreamingResponse:
        return TCPFlowProtectionResourceWithStreamingResponse(self._advanced_tcp_protection.tcp_flow_protection)

    @cached_property
    def status(self) -> StatusResourceWithStreamingResponse:
        return StatusResourceWithStreamingResponse(self._advanced_tcp_protection.status)


class AsyncAdvancedTCPProtectionResourceWithStreamingResponse:
    def __init__(self, advanced_tcp_protection: AsyncAdvancedTCPProtectionResource) -> None:
        self._advanced_tcp_protection = advanced_tcp_protection

    @cached_property
    def allowlist(self) -> AsyncAllowlistResourceWithStreamingResponse:
        return AsyncAllowlistResourceWithStreamingResponse(self._advanced_tcp_protection.allowlist)

    @cached_property
    def prefixes(self) -> AsyncPrefixesResourceWithStreamingResponse:
        return AsyncPrefixesResourceWithStreamingResponse(self._advanced_tcp_protection.prefixes)

    @cached_property
    def syn_protection(self) -> AsyncSynProtectionResourceWithStreamingResponse:
        return AsyncSynProtectionResourceWithStreamingResponse(self._advanced_tcp_protection.syn_protection)

    @cached_property
    def tcp_flow_protection(self) -> AsyncTCPFlowProtectionResourceWithStreamingResponse:
        return AsyncTCPFlowProtectionResourceWithStreamingResponse(self._advanced_tcp_protection.tcp_flow_protection)

    @cached_property
    def status(self) -> AsyncStatusResourceWithStreamingResponse:
        return AsyncStatusResourceWithStreamingResponse(self._advanced_tcp_protection.status)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .operation_list_params import OperationListParams as OperationListParams
from .operation_get_response import OperationGetResponse as OperationGetResponse
from .operation_list_response import OperationListResponse as OperationListResponse
from .operation_update_params import OperationUpdateParams as OperationUpdateParams
from .operation_delete_response import OperationDeleteResponse as OperationDeleteResponse
from .operation_update_response import OperationUpdateResponse as OperationUpdateResponse
from .operation_bulk_edit_params import OperationBulkEditParams as OperationBulkEditParams
from .operation_bulk_edit_response import OperationBulkEditResponse as OperationBulkEditResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, cast

import httpx

from ...._types import Body, Query, Headers, NotGiven, FileTypes, not_given
from ...._utils import extract_files, path_template, maybe_transform, deepcopy_minimal, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.ai.finetunes import asset_create_params
from ....types.ai.finetunes.asset_create_response import AssetCreateResponse

__all__ = ["AssetsResource", "AsyncAssetsResource"]


class AssetsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AssetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AssetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AssetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AssetsResourceWithStreamingResponse(self)

    def create(
        self,
        finetune_id: str,
        *,
        account_id: str,
        file: FileTypes,
        file_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AssetCreateResponse:
        """
        Uploads training data assets for a Workers AI fine-tuning job.

        Args:
          file: File to upload

          file_name: Name of the file (adapter_config.json or adapter_model.safetensors)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not finetune_id:
            raise ValueError(f"Expected a non-empty value for `finetune_id` but received {finetune_id!r}")
        body = deepcopy_minimal(
            {
                "file": file,
                "file_name": file_name,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            path_template(
                "/accounts/{account_id}/ai/finetunes/{finetune_id}/finetune-assets",
                account_id=account_id,
                finetune_id=finetune_id,
            ),
            body=maybe_transform(body, asset_create_params.AssetCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AssetCreateResponse,
        )


class AsyncAssetsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAssetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAssetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAssetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAssetsResourceWithStreamingResponse(self)

    async def create(
        self,
        finetune_id: str,
        *,
        account_id: str,
        file: FileTypes,
        file_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AssetCreateResponse:
        """
        Uploads training data assets for a Workers AI fine-tuning job.

        Args:
          file: File to upload

          file_name: Name of the file (adapter_config.json or adapter_model.safetensors)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not finetune_id:
            raise ValueError(f"Expected a non-empty value for `finetune_id` but received {finetune_id!r}")
        body = deepcopy_minimal(
            {
                "file": file,
                "file_name": file_name,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            path_template(
                "/accounts/{account_id}/ai/finetunes/{finetune_id}/finetune-assets",
                account_id=account_id,
                finetune_id=finetune_id,
            ),
            body=await async_maybe_transform(body, asset_create_params.AssetCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AssetCreateResponse,
        )


class AssetsResourceWithRawResponse:
    def __init__(self, assets: AssetsResource) -> None:
        self._assets = assets

        self.create = to_raw_response_wrapper(
            assets.create,
        )


class AsyncAssetsResourceWithRawResponse:
    def __init__(self, assets: AsyncAssetsResource) -> None:
        self._assets = assets

        self.create = async_to_raw_response_wrapper(
            assets.create,
        )


class AssetsResourceWithStreamingResponse:
    def __init__(self, assets: AssetsResource) -> None:
        self._assets = assets

        self.create = to_streamed_response_wrapper(
            assets.create,
        )


class AsyncAssetsResourceWithStreamingResponse:
    def __init__(self, assets: AsyncAssetsResource) -> None:
        self._assets = assets

        self.create = async_to_streamed_response_wrapper(
            assets.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CertificatePackEditParams"]


class CertificatePackEditParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    cloudflare_branding: bool
    """Whether or not to add Cloudflare Branding for the order.

    This will add a subdomain of sni.cloudflaressl.com as the Common Name if set to
    true.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel

__all__ = ["EmailListRule", "EmailList"]


class EmailList(BaseModel):
    id: str
    """The ID of a previously created email list."""


class EmailListRule(BaseModel):
    """Matches an email address from a list."""

    email_list: EmailList



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.zero_trust.identity_providers.saml_certificate_create_response import SAMLCertificateCreateResponse

__all__ = ["SAMLCertificateResource", "AsyncSAMLCertificateResource"]


class SAMLCertificateResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SAMLCertificateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SAMLCertificateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SAMLCertificateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SAMLCertificateResourceWithStreamingResponse(self)

    def create(
        self,
        identity_provider_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SAMLCertificateCreateResponse]:
        """
        Creates a new SAML encryption certificate set and assigns it to the specified
        SAML Identity Provider. This endpoint is idempotent - if the IdP already has a
        certificate set assigned, the existing certificate set is returned with a 200
        status.

        **Workflow for enabling SAML encryption:**

        1. Call this endpoint to create and assign a certificate set to the IdP
        2. Update the IdP configuration (PUT `/identity_providers/{id}`) with:
           - `config.enable_encryption: true`
           - `saml_certificate_set_id: <uid from step 1>`
        3. Configure the certificate's public key in your external SAML Identity
           Provider

        Args:
          account_id: Identifier.

          identity_provider_id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not identity_provider_id:
            raise ValueError(
                f"Expected a non-empty value for `identity_provider_id` but received {identity_provider_id!r}"
            )
        return self._post(
            path_template(
                "/accounts/{account_id}/access/identity_providers/{identity_provider_id}/saml_certificate",
                account_id=account_id,
                identity_provider_id=identity_provider_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SAMLCertificateCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SAMLCertificateCreateResponse]], ResultWrapper[SAMLCertificateCreateResponse]),
        )


class AsyncSAMLCertificateResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSAMLCertificateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSAMLCertificateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSAMLCertificateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSAMLCertificateResourceWithStreamingResponse(self)

    async def create(
        self,
        identity_provider_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SAMLCertificateCreateResponse]:
        """
        Creates a new SAML encryption certificate set and assigns it to the specified
        SAML Identity Provider. This endpoint is idempotent - if the IdP already has a
        certificate set assigned, the existing certificate set is returned with a 200
        status.

        **Workflow for enabling SAML encryption:**

        1. Call this endpoint to create and assign a certificate set to the IdP
        2. Update the IdP configuration (PUT `/identity_providers/{id}`) with:
           - `config.enable_encryption: true`
           - `saml_certificate_set_id: <uid from step 1>`
        3. Configure the certificate's public key in your external SAML Identity
           Provider

        Args:
          account_id: Identifier.

          identity_provider_id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not identity_provider_id:
            raise ValueError(
                f"Expected a non-empty value for `identity_provider_id` but received {identity_provider_id!r}"
            )
        return await self._post(
            path_template(
                "/accounts/{account_id}/access/identity_providers/{identity_provider_id}/saml_certificate",
                account_id=account_id,
                identity_provider_id=identity_provider_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SAMLCertificateCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SAMLCertificateCreateResponse]], ResultWrapper[SAMLCertificateCreateResponse]),
        )


class SAMLCertificateResourceWithRawResponse:
    def __init__(self, saml_certificate: SAMLCertificateResource) -> None:
        self._saml_certificate = saml_certificate

        self.create = to_raw_response_wrapper(
            saml_certificate.create,
        )


class AsyncSAMLCertificateResourceWithRawResponse:
    def __init__(self, saml_certificate: AsyncSAMLCertificateResource) -> None:
        self._saml_certificate = saml_certificate

        self.create = async_to_raw_response_wrapper(
            saml_certificate.create,
        )


class SAMLCertificateResourceWithStreamingResponse:
    def __init__(self, saml_certificate: SAMLCertificateResource) -> None:
        self._saml_certificate = saml_certificate

        self.create = to_streamed_response_wrapper(
            saml_certificate.create,
        )


class AsyncSAMLCertificateResourceWithStreamingResponse:
    def __init__(self, saml_certificate: AsyncSAMLCertificateResource) -> None:
        self._saml_certificate = saml_certificate

        self.create = async_to_streamed_response_wrapper(
            saml_certificate.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .rate_limits import (
    RateLimitsResource,
    AsyncRateLimitsResource,
    RateLimitsResourceWithRawResponse,
    AsyncRateLimitsResourceWithRawResponse,
    RateLimitsResourceWithStreamingResponse,
    AsyncRateLimitsResourceWithStreamingResponse,
)

__all__ = [
    "RateLimitsResource",
    "AsyncRateLimitsResource",
    "RateLimitsResourceWithRawResponse",
    "AsyncRateLimitsResourceWithRawResponse",
    "RateLimitsResourceWithStreamingResponse",
    "AsyncRateLimitsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Device", "User"]


class User(BaseModel):
    id: Optional[str] = None
    """UUID."""

    email: Optional[str] = None
    """The contact email address of the user."""

    name: Optional[str] = None
    """The enrolled device user's name."""


class Device(BaseModel):
    id: Optional[str] = None
    """Registration ID.

    Equal to Device ID except for accounts which enabled
    [multi-user mode](https://developers.cloudflare.com/cloudflare-one/connections/connect-devices/warp/deployment/mdm-deployment/windows-multiuser/).
    """

    created: Optional[datetime] = None
    """When the device was created."""

    deleted: Optional[bool] = None
    """True if the device was deleted."""

    device_type: Optional[Literal["windows", "mac", "linux", "android", "ios", "chromeos"]] = None

    ip: Optional[str] = None
    """IPv4 or IPv6 address."""

    key: Optional[str] = None
    """The device's public key."""

    last_seen: Optional[datetime] = None
    """When the device last connected to Cloudflare services."""

    mac_address: Optional[str] = None
    """The device mac address."""

    manufacturer: Optional[str] = None
    """The device manufacturer name."""

    model: Optional[str] = None
    """The device model name."""

    name: Optional[str] = None
    """The device name."""

    os_distro_name: Optional[str] = None
    """The Linux distro name."""

    os_distro_revision: Optional[str] = None
    """The Linux distro revision."""

    os_version: Optional[str] = None
    """The operating system version."""

    os_version_extra: Optional[str] = None
    """Additional operating system version details.

    For Windows, the UBR (Update Build Revision). For Mac or iOS, the Product
    Version Extra. For Linux, the distribution name and version.
    """

    revoked_at: Optional[datetime] = None
    """When the device was revoked."""

    serial_number: Optional[str] = None
    """The device serial number."""

    updated: Optional[datetime] = None
    """When the device was updated."""

    user: Optional[User] = None

    version: Optional[str] = None
    """The WARP client version."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["GroupEditParams"]


class GroupEditParams(TypedDict, total=False):
    zone_id: Required[str]
    """Defines an identifier of a schema."""

    package_id: Required[str]
    """Defines the unique identifier of a WAF package."""

    mode: Literal["on", "off"]
    """Defines the state of the rules contained in the rule group.

    When `on`, the rules in the group are configurable/usable.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SSLRecommender"]


class SSLRecommender(BaseModel):
    """
    Enrollment in the SSL/TLS Recommender service which tries to detect and recommend (by sending periodic emails) the most secure SSL/TLS setting your origin servers support.
    """

    id: Optional[Literal["ssl_recommender"]] = None
    """Enrollment value for SSL/TLS Recommender."""

    enabled: Optional[bool] = None
    """ssl-recommender enrollment setting."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .prebuilt_policy_list_params import PrebuiltPolicyListParams as PrebuiltPolicyListParams
from .prebuilt_policy_list_response import PrebuiltPolicyListResponse as PrebuiltPolicyListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["WatermarkCreateParams"]


class WatermarkCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    name: str
    """A short description of the watermark profile."""

    opacity: float
    """The translucency of the image.

    A value of `0.0` makes the image completely transparent, and `1.0` makes the
    image completely opaque. Note that if the image is already semi-transparent,
    setting this to `1.0` will not make the image completely opaque.
    """

    padding: float
    """
    The whitespace between the adjacent edges (determined by position) of the video
    and the image. `0.0` indicates no padding, and `1.0` indicates a fully padded
    video width or length, as determined by the algorithm.
    """

    position: str
    """The location of the image.

    Valid positions are: `upperRight`, `upperLeft`, `lowerLeft`, `lowerRight`, and
    `center`. Note that `center` ignores the `padding` parameter.
    """

    scale: float
    """The size of the image relative to the overall size of the video.

    This parameter will adapt to horizontal and vertical videos automatically. `0.0`
    indicates no scaling (use the size of the image as-is), and `1.0 `fills the
    entire video.
    """

    url: str
    """URL of the watermark image to copy."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["SecretListParams"]


class SecretListParams(TypedDict, total=False):
    account_id: Required[str]
    """Account Identifier"""

    direction: Literal["asc", "desc"]
    """Direction to sort objects"""

    order: Literal["name", "comment", "created", "modified", "status"]
    """Order secrets by values in the given field"""

    page: int
    """Page number"""

    per_page: int
    """Number of objects to return per page"""

    scopes: Iterable[SequenceNotStr[str]]
    """Only secrets with the given scopes will be returned"""

    search: str
    """Search secrets using a filter string, filtering across name and comment"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel
from .ip_network import IPNetwork

__all__ = ["DOHEndpoint"]


class DOHEndpoint(BaseModel):
    enabled: Optional[bool] = None
    """Indicate whether the DOH endpoint is enabled for this location."""

    networks: Optional[List[IPNetwork]] = None
    """Specify the list of allowed source IP network ranges for this endpoint.

    When the list is empty, the endpoint allows all source IPs. The list takes
    effect only if the endpoint is enabled for this location.
    """

    require_token: Optional[bool] = None
    """Specify whether the DOH endpoint requires user identity authentication."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["SeverityQueryParam"]

SeverityQueryParam: TypeAlias = Literal["low", "moderate", "critical"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PagePreviewParams"]


class PagePreviewParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    custom_html: Required[str]
    """Only available for the Waiting Room Advanced subscription.

    This is a template html file that will be rendered at the edge. If no
    custom_page_html is provided, the default waiting room will be used. The
    template is based on mustache ( https://mustache.github.io/ ). There are several
    variables that are evaluated by the Cloudflare edge:

    1. {{`waitTimeKnown`}} Acts like a boolean value that indicates the behavior to
       take when wait time is not available, for instance when queue_all is
       **true**.
    2. {{`waitTimeFormatted`}} Estimated wait time for the user. For example, five
       minutes. Alternatively, you can use:
    3. {{`waitTime`}} Number of minutes of estimated wait for a user.
    4. {{`waitTimeHours`}} Number of hours of estimated wait for a user
       (`Math.floor(waitTime/60)`).
    5. {{`waitTimeHourMinutes`}} Number of minutes above the `waitTimeHours` value
       (`waitTime%60`).
    6. {{`queueIsFull`}} Changes to **true** when no more people can be added to the
       queue.

    To view the full list of variables, look at the `cfWaitingRoom` object described
    under the `json_response_enabled` property in other Waiting Room API calls.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ....._compat import cached_property
from .rules.rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from ....._resource import SyncAPIResource, AsyncAPIResource
from .filters.filters import (
    FiltersResource,
    AsyncFiltersResource,
    FiltersResourceWithRawResponse,
    AsyncFiltersResourceWithRawResponse,
    FiltersResourceWithStreamingResponse,
    AsyncFiltersResourceWithStreamingResponse,
)

__all__ = ["TCPFlowProtectionResource", "AsyncTCPFlowProtectionResource"]


class TCPFlowProtectionResource(SyncAPIResource):
    @cached_property
    def filters(self) -> FiltersResource:
        return FiltersResource(self._client)

    @cached_property
    def rules(self) -> RulesResource:
        return RulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> TCPFlowProtectionResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TCPFlowProtectionResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TCPFlowProtectionResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TCPFlowProtectionResourceWithStreamingResponse(self)


class AsyncTCPFlowProtectionResource(AsyncAPIResource):
    @cached_property
    def filters(self) -> AsyncFiltersResource:
        return AsyncFiltersResource(self._client)

    @cached_property
    def rules(self) -> AsyncRulesResource:
        return AsyncRulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTCPFlowProtectionResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTCPFlowProtectionResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTCPFlowProtectionResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTCPFlowProtectionResourceWithStreamingResponse(self)


class TCPFlowProtectionResourceWithRawResponse:
    def __init__(self, tcp_flow_protection: TCPFlowProtectionResource) -> None:
        self._tcp_flow_protection = tcp_flow_protection

    @cached_property
    def filters(self) -> FiltersResourceWithRawResponse:
        return FiltersResourceWithRawResponse(self._tcp_flow_protection.filters)

    @cached_property
    def rules(self) -> RulesResourceWithRawResponse:
        return RulesResourceWithRawResponse(self._tcp_flow_protection.rules)


class AsyncTCPFlowProtectionResourceWithRawResponse:
    def __init__(self, tcp_flow_protection: AsyncTCPFlowProtectionResource) -> None:
        self._tcp_flow_protection = tcp_flow_protection

    @cached_property
    def filters(self) -> AsyncFiltersResourceWithRawResponse:
        return AsyncFiltersResourceWithRawResponse(self._tcp_flow_protection.filters)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithRawResponse:
        return AsyncRulesResourceWithRawResponse(self._tcp_flow_protection.rules)


class TCPFlowProtectionResourceWithStreamingResponse:
    def __init__(self, tcp_flow_protection: TCPFlowProtectionResource) -> None:
        self._tcp_flow_protection = tcp_flow_protection

    @cached_property
    def filters(self) -> FiltersResourceWithStreamingResponse:
        return FiltersResourceWithStreamingResponse(self._tcp_flow_protection.filters)

    @cached_property
    def rules(self) -> RulesResourceWithStreamingResponse:
        return RulesResourceWithStreamingResponse(self._tcp_flow_protection.rules)


class AsyncTCPFlowProtectionResourceWithStreamingResponse:
    def __init__(self, tcp_flow_protection: AsyncTCPFlowProtectionResource) -> None:
        self._tcp_flow_protection = tcp_flow_protection

    @cached_property
    def filters(self) -> AsyncFiltersResourceWithStreamingResponse:
        return AsyncFiltersResourceWithStreamingResponse(self._tcp_flow_protection.filters)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithStreamingResponse:
        return AsyncRulesResourceWithStreamingResponse(self._tcp_flow_protection.rules)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.firewall import access_rule_edit_params, access_rule_list_params, access_rule_create_params
from ...types.firewall.access_rule_get_response import AccessRuleGetResponse
from ...types.firewall.access_rule_edit_response import AccessRuleEditResponse
from ...types.firewall.access_rule_list_response import AccessRuleListResponse
from ...types.firewall.access_rule_create_response import AccessRuleCreateResponse
from ...types.firewall.access_rule_delete_response import AccessRuleDeleteResponse

__all__ = ["AccessRulesResource", "AsyncAccessRulesResource"]


class AccessRulesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AccessRulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AccessRulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccessRulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AccessRulesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        configuration: access_rule_create_params.Configuration,
        mode: Literal["block", "challenge", "whitelist", "js_challenge", "managed_challenge"],
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        notes: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccessRuleCreateResponse:
        """Creates a new IP Access rule for an account or zone.

        The rule will apply to all
        zones in the account or zone.

        Note: To create an IP Access rule that applies to a single zone, refer to the
        [IP Access rules for a zone](#ip-access-rules-for-a-zone) endpoints.

        Args:
          configuration: The rule configuration.

          mode: The action to apply to a matched request.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          notes: An informative summary of the rule, typically used as a reminder or explanation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._post(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/firewall/access_rules/rules",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=maybe_transform(
                {
                    "configuration": configuration,
                    "mode": mode,
                    "notes": notes,
                },
                access_rule_create_params.AccessRuleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AccessRuleCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[AccessRuleCreateResponse], ResultWrapper[AccessRuleCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        configuration: access_rule_list_params.Configuration | Omit = omit,
        direction: Literal["asc", "desc"] | Omit = omit,
        match: Literal["any", "all"] | Omit = omit,
        mode: Literal["block", "challenge", "whitelist", "js_challenge", "managed_challenge"] | Omit = omit,
        notes: str | Omit = omit,
        order: Literal["configuration.target", "configuration.value", "mode"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[AccessRuleListResponse]:
        """Fetches IP Access rules of an account or zone.

        These rules apply to all the
        zones in the account or zone. You can filter the results using several optional
        parameters.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          direction: Defines the direction used to sort returned rules.

          match: Defines the search requirements. When set to `all`, all the search requirements
              must match. When set to `any`, only one of the search requirements has to match.

          mode: The action to apply to a matched request.

          notes: Defines the string to search for in the notes of existing IP Access rules.
              Notes: For example, the string 'attack' would match IP Access rules with notes
              'Attack 26/02' and 'Attack 27/02'. The search is case insensitive.

          order: Defines the field used to sort returned rules.

          page: Defines the requested page within paginated list of results.

          per_page: Defines the maximum number of results requested.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/firewall/access_rules/rules",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=SyncV4PagePaginationArray[AccessRuleListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "configuration": configuration,
                        "direction": direction,
                        "match": match,
                        "mode": mode,
                        "notes": notes,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                    },
                    access_rule_list_params.AccessRuleListParams,
                ),
            ),
            model=AccessRuleListResponse,
        )

    def delete(
        self,
        rule_id: str,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AccessRuleDeleteResponse]:
        """
        Deletes an existing IP Access rule defined.

        Note: This operation will affect all zones in the account or zone.

        Args:
          rule_id: Unique identifier for a rule.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._delete(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/firewall/access_rules/rules/{rule_id}",
                rule_id=rule_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AccessRuleDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AccessRuleDeleteResponse]], ResultWrapper[AccessRuleDeleteResponse]),
        )

    def edit(
        self,
        rule_id: str,
        *,
        configuration: access_rule_edit_params.Configuration,
        mode: Literal["block", "challenge", "whitelist", "js_challenge", "managed_challenge"],
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        notes: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccessRuleEditResponse:
        """
        Updates an IP Access rule defined.

        Note: This operation will affect all zones in the account or zone.

        Args:
          rule_id: Unique identifier for a rule.

          configuration: The rule configuration.

          mode: The action to apply to a matched request.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          notes: An informative summary of the rule, typically used as a reminder or explanation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._patch(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/firewall/access_rules/rules/{rule_id}",
                rule_id=rule_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=maybe_transform(
                {
                    "configuration": configuration,
                    "mode": mode,
                    "notes": notes,
                },
                access_rule_edit_params.AccessRuleEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AccessRuleEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[AccessRuleEditResponse], ResultWrapper[AccessRuleEditResponse]),
        )

    def get(
        self,
        rule_id: str,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccessRuleGetResponse:
        """
        Fetches the details of an IP Access rule defined.

        Args:
          rule_id: Unique identifier for a rule.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/firewall/access_rules/rules/{rule_id}",
                rule_id=rule_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AccessRuleGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[AccessRuleGetResponse], ResultWrapper[AccessRuleGetResponse]),
        )


class AsyncAccessRulesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAccessRulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAccessRulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccessRulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAccessRulesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        configuration: access_rule_create_params.Configuration,
        mode: Literal["block", "challenge", "whitelist", "js_challenge", "managed_challenge"],
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        notes: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccessRuleCreateResponse:
        """Creates a new IP Access rule for an account or zone.

        The rule will apply to all
        zones in the account or zone.

        Note: To create an IP Access rule that applies to a single zone, refer to the
        [IP Access rules for a zone](#ip-access-rules-for-a-zone) endpoints.

        Args:
          configuration: The rule configuration.

          mode: The action to apply to a matched request.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          notes: An informative summary of the rule, typically used as a reminder or explanation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._post(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/firewall/access_rules/rules",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=await async_maybe_transform(
                {
                    "configuration": configuration,
                    "mode": mode,
                    "notes": notes,
                },
                access_rule_create_params.AccessRuleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AccessRuleCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[AccessRuleCreateResponse], ResultWrapper[AccessRuleCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        configuration: access_rule_list_params.Configuration | Omit = omit,
        direction: Literal["asc", "desc"] | Omit = omit,
        match: Literal["any", "all"] | Omit = omit,
        mode: Literal["block", "challenge", "whitelist", "js_challenge", "managed_challenge"] | Omit = omit,
        notes: str | Omit = omit,
        order: Literal["configuration.target", "configuration.value", "mode"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AccessRuleListResponse, AsyncV4PagePaginationArray[AccessRuleListResponse]]:
        """Fetches IP Access rules of an account or zone.

        These rules apply to all the
        zones in the account or zone. You can filter the results using several optional
        parameters.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          direction: Defines the direction used to sort returned rules.

          match: Defines the search requirements. When set to `all`, all the search requirements
              must match. When set to `any`, only one of the search requirements has to match.

          mode: The action to apply to a matched request.

          notes: Defines the string to search for in the notes of existing IP Access rules.
              Notes: For example, the string 'attack' would match IP Access rules with notes
              'Attack 26/02' and 'Attack 27/02'. The search is case insensitive.

          order: Defines the field used to sort returned rules.

          page: Defines the requested page within paginated list of results.

          per_page: Defines the maximum number of results requested.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/firewall/access_rules/rules",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=AsyncV4PagePaginationArray[AccessRuleListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "configuration": configuration,
                        "direction": direction,
                        "match": match,
                        "mode": mode,
                        "notes": notes,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                    },
                    access_rule_list_params.AccessRuleListParams,
                ),
            ),
            model=AccessRuleListResponse,
        )

    async def delete(
        self,
        rule_id: str,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AccessRuleDeleteResponse]:
        """
        Deletes an existing IP Access rule defined.

        Note: This operation will affect all zones in the account or zone.

        Args:
          rule_id: Unique identifier for a rule.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._delete(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/firewall/access_rules/rules/{rule_id}",
                rule_id=rule_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AccessRuleDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AccessRuleDeleteResponse]], ResultWrapper[AccessRuleDeleteResponse]),
        )

    async def edit(
        self,
        rule_id: str,
        *,
        configuration: access_rule_edit_params.Configuration,
        mode: Literal["block", "challenge", "whitelist", "js_challenge", "managed_challenge"],
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        notes: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccessRuleEditResponse:
        """
        Updates an IP Access rule defined.

        Note: This operation will affect all zones in the account or zone.

        Args:
          rule_id: Unique identifier for a rule.

          configuration: The rule configuration.

          mode: The action to apply to a matched request.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          notes: An informative summary of the rule, typically used as a reminder or explanation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._patch(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/firewall/access_rules/rules/{rule_id}",
                rule_id=rule_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=await async_maybe_transform(
                {
                    "configuration": configuration,
                    "mode": mode,
                    "notes": notes,
                },
                access_rule_edit_params.AccessRuleEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AccessRuleEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[AccessRuleEditResponse], ResultWrapper[AccessRuleEditResponse]),
        )

    async def get(
        self,
        rule_id: str,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccessRuleGetResponse:
        """
        Fetches the details of an IP Access rule defined.

        Args:
          rule_id: Unique identifier for a rule.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._get(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/firewall/access_rules/rules/{rule_id}",
                rule_id=rule_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AccessRuleGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[AccessRuleGetResponse], ResultWrapper[AccessRuleGetResponse]),
        )


class AccessRulesResourceWithRawResponse:
    def __init__(self, access_rules: AccessRulesResource) -> None:
        self._access_rules = access_rules

        self.create = to_raw_response_wrapper(
            access_rules.create,
        )
        self.list = to_raw_response_wrapper(
            access_rules.list,
        )
        self.delete = to_raw_response_wrapper(
            access_rules.delete,
        )
        self.edit = to_raw_response_wrapper(
            access_rules.edit,
        )
        self.get = to_raw_response_wrapper(
            access_rules.get,
        )


class AsyncAccessRulesResourceWithRawResponse:
    def __init__(self, access_rules: AsyncAccessRulesResource) -> None:
        self._access_rules = access_rules

        self.create = async_to_raw_response_wrapper(
            access_rules.create,
        )
        self.list = async_to_raw_response_wrapper(
            access_rules.list,
        )
        self.delete = async_to_raw_response_wrapper(
            access_rules.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            access_rules.edit,
        )
        self.get = async_to_raw_response_wrapper(
            access_rules.get,
        )


class AccessRulesResourceWithStreamingResponse:
    def __init__(self, access_rules: AccessRulesResource) -> None:
        self._access_rules = access_rules

        self.create = to_streamed_response_wrapper(
            access_rules.create,
        )
        self.list = to_streamed_response_wrapper(
            access_rules.list,
        )
        self.delete = to_streamed_response_wrapper(
            access_rules.delete,
        )
        self.edit = to_streamed_response_wrapper(
            access_rules.edit,
        )
        self.get = to_streamed_response_wrapper(
            access_rules.get,
        )


class AsyncAccessRulesResourceWithStreamingResponse:
    def __init__(self, access_rules: AsyncAccessRulesResource) -> None:
        self._access_rules = access_rules

        self.create = async_to_streamed_response_wrapper(
            access_rules.create,
        )
        self.list = async_to_streamed_response_wrapper(
            access_rules.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            access_rules.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            access_rules.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            access_rules.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["NamespaceListParams"]


class NamespaceListParams(TypedDict, total=False):
    account_id: Required[str]
    """Use this to identify the account."""

    page_size: int
    """Maximum number of namespaces to return per page. Defaults to 100, maximum 1000."""

    page_token: str
    """
    Opaque pagination token from a previous response. Use this to fetch the next
    page of results.
    """

    parent: str
    """Parent namespace to filter by.

    Only returns direct children of this namespace. For nested namespaces, use %1F
    as separator (e.g., "bronze%1Fanalytics"). Omit this parameter to list top-level
    namespaces.
    """

    return_details: bool
    """
    Whether to include additional metadata (timestamps). When true, response
    includes created_at and updated_at arrays.
    """

    return_uuids: bool
    """
    Whether to include namespace UUIDs in the response. Set to true to receive the
    namespace_uuids array.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast
from typing_extensions import Literal, overload

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import is_given, path_template, required_args, maybe_transform, strip_not_given, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.r2.buckets import sippy_update_params
from ....types.r2.buckets.sippy import Sippy
from ....types.r2.buckets.sippy_delete_response import SippyDeleteResponse

__all__ = ["SippyResource", "AsyncSippyResource"]


class SippyResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SippyResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SippyResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SippyResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SippyResourceWithStreamingResponse(self)

    @overload
    def update(
        self,
        bucket_name: str,
        *,
        account_id: str,
        destination: sippy_update_params.R2EnableSippyAwsDestination | Omit = omit,
        source: sippy_update_params.R2EnableSippyAwsSource | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Sippy:
        """
        Sets configuration for Sippy for an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          destination: R2 bucket to copy objects to.

          source: AWS S3 bucket to copy objects from.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        bucket_name: str,
        *,
        account_id: str,
        destination: sippy_update_params.R2EnableSippyGcsDestination | Omit = omit,
        source: sippy_update_params.R2EnableSippyGcsSource | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Sippy:
        """
        Sets configuration for Sippy for an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          destination: R2 bucket to copy objects to.

          source: GCS bucket to copy objects from.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        bucket_name: str,
        *,
        account_id: str,
        destination: sippy_update_params.R2EnableSippyS3Destination | Omit = omit,
        source: sippy_update_params.R2EnableSippyS3Source | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Sippy:
        """
        Sets configuration for Sippy for an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          destination: R2 bucket to copy objects to.

          source: General S3-compatible provider to copy objects from.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id"])
    def update(
        self,
        bucket_name: str,
        *,
        account_id: str,
        destination: sippy_update_params.R2EnableSippyAwsDestination
        | sippy_update_params.R2EnableSippyGcsDestination
        | sippy_update_params.R2EnableSippyS3Destination
        | Omit = omit,
        source: sippy_update_params.R2EnableSippyAwsSource
        | sippy_update_params.R2EnableSippyGcsSource
        | sippy_update_params.R2EnableSippyS3Source
        | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Sippy:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return self._put(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/sippy", account_id=account_id, bucket_name=bucket_name
            ),
            body=maybe_transform(
                {
                    "destination": destination,
                    "source": source,
                },
                sippy_update_params.SippyUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Sippy]._unwrapper,
            ),
            cast_to=cast(Type[Sippy], ResultWrapper[Sippy]),
        )

    def delete(
        self,
        bucket_name: str,
        *,
        account_id: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SippyDeleteResponse:
        """
        Disables Sippy on this bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return self._delete(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/sippy", account_id=account_id, bucket_name=bucket_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SippyDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[SippyDeleteResponse], ResultWrapper[SippyDeleteResponse]),
        )

    def get(
        self,
        bucket_name: str,
        *,
        account_id: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Sippy:
        """
        Gets configuration for Sippy for an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return self._get(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/sippy", account_id=account_id, bucket_name=bucket_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Sippy]._unwrapper,
            ),
            cast_to=cast(Type[Sippy], ResultWrapper[Sippy]),
        )


class AsyncSippyResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSippyResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSippyResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSippyResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSippyResourceWithStreamingResponse(self)

    @overload
    async def update(
        self,
        bucket_name: str,
        *,
        account_id: str,
        destination: sippy_update_params.R2EnableSippyAwsDestination | Omit = omit,
        source: sippy_update_params.R2EnableSippyAwsSource | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Sippy:
        """
        Sets configuration for Sippy for an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          destination: R2 bucket to copy objects to.

          source: AWS S3 bucket to copy objects from.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        bucket_name: str,
        *,
        account_id: str,
        destination: sippy_update_params.R2EnableSippyGcsDestination | Omit = omit,
        source: sippy_update_params.R2EnableSippyGcsSource | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Sippy:
        """
        Sets configuration for Sippy for an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          destination: R2 bucket to copy objects to.

          source: GCS bucket to copy objects from.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        bucket_name: str,
        *,
        account_id: str,
        destination: sippy_update_params.R2EnableSippyS3Destination | Omit = omit,
        source: sippy_update_params.R2EnableSippyS3Source | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Sippy:
        """
        Sets configuration for Sippy for an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          destination: R2 bucket to copy objects to.

          source: General S3-compatible provider to copy objects from.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id"])
    async def update(
        self,
        bucket_name: str,
        *,
        account_id: str,
        destination: sippy_update_params.R2EnableSippyAwsDestination
        | sippy_update_params.R2EnableSippyGcsDestination
        | sippy_update_params.R2EnableSippyS3Destination
        | Omit = omit,
        source: sippy_update_params.R2EnableSippyAwsSource
        | sippy_update_params.R2EnableSippyGcsSource
        | sippy_update_params.R2EnableSippyS3Source
        | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Sippy:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return await self._put(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/sippy", account_id=account_id, bucket_name=bucket_name
            ),
            body=await async_maybe_transform(
                {
                    "destination": destination,
                    "source": source,
                },
                sippy_update_params.SippyUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Sippy]._unwrapper,
            ),
            cast_to=cast(Type[Sippy], ResultWrapper[Sippy]),
        )

    async def delete(
        self,
        bucket_name: str,
        *,
        account_id: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SippyDeleteResponse:
        """
        Disables Sippy on this bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return await self._delete(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/sippy", account_id=account_id, bucket_name=bucket_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SippyDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[SippyDeleteResponse], ResultWrapper[SippyDeleteResponse]),
        )

    async def get(
        self,
        bucket_name: str,
        *,
        account_id: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Sippy:
        """
        Gets configuration for Sippy for an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return await self._get(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/sippy", account_id=account_id, bucket_name=bucket_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Sippy]._unwrapper,
            ),
            cast_to=cast(Type[Sippy], ResultWrapper[Sippy]),
        )


class SippyResourceWithRawResponse:
    def __init__(self, sippy: SippyResource) -> None:
        self._sippy = sippy

        self.update = to_raw_response_wrapper(
            sippy.update,
        )
        self.delete = to_raw_response_wrapper(
            sippy.delete,
        )
        self.get = to_raw_response_wrapper(
            sippy.get,
        )


class AsyncSippyResourceWithRawResponse:
    def __init__(self, sippy: AsyncSippyResource) -> None:
        self._sippy = sippy

        self.update = async_to_raw_response_wrapper(
            sippy.update,
        )
        self.delete = async_to_raw_response_wrapper(
            sippy.delete,
        )
        self.get = async_to_raw_response_wrapper(
            sippy.get,
        )


class SippyResourceWithStreamingResponse:
    def __init__(self, sippy: SippyResource) -> None:
        self._sippy = sippy

        self.update = to_streamed_response_wrapper(
            sippy.update,
        )
        self.delete = to_streamed_response_wrapper(
            sippy.delete,
        )
        self.get = to_streamed_response_wrapper(
            sippy.get,
        )


class AsyncSippyResourceWithStreamingResponse:
    def __init__(self, sippy: AsyncSippyResource) -> None:
        self._sippy = sippy

        self.update = async_to_streamed_response_wrapper(
            sippy.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            sippy.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            sippy.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["ConfigGetParams"]


class ConfigGetParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    ids: Required[Iterable[int]]
    """Comma separated list of Zaraz configuration IDs."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from .configs import (
    ConfigsResource,
    AsyncConfigsResource,
    ConfigsResourceWithRawResponse,
    AsyncConfigsResourceWithRawResponse,
    ConfigsResourceWithStreamingResponse,
    AsyncConfigsResourceWithStreamingResponse,
)
from .vpc_flows import (
    VPCFlowsResource,
    AsyncVPCFlowsResource,
    VPCFlowsResourceWithRawResponse,
    AsyncVPCFlowsResourceWithRawResponse,
    VPCFlowsResourceWithStreamingResponse,
    AsyncVPCFlowsResourceWithStreamingResponse,
)
from .magic_network_monitoring import (
    MagicNetworkMonitoringResource,
    AsyncMagicNetworkMonitoringResource,
    MagicNetworkMonitoringResourceWithRawResponse,
    AsyncMagicNetworkMonitoringResourceWithRawResponse,
    MagicNetworkMonitoringResourceWithStreamingResponse,
    AsyncMagicNetworkMonitoringResourceWithStreamingResponse,
)

__all__ = [
    "VPCFlowsResource",
    "AsyncVPCFlowsResource",
    "VPCFlowsResourceWithRawResponse",
    "AsyncVPCFlowsResourceWithRawResponse",
    "VPCFlowsResourceWithStreamingResponse",
    "AsyncVPCFlowsResourceWithStreamingResponse",
    "ConfigsResource",
    "AsyncConfigsResource",
    "ConfigsResourceWithRawResponse",
    "AsyncConfigsResourceWithRawResponse",
    "ConfigsResourceWithStreamingResponse",
    "AsyncConfigsResourceWithStreamingResponse",
    "RulesResource",
    "AsyncRulesResource",
    "RulesResourceWithRawResponse",
    "AsyncRulesResourceWithRawResponse",
    "RulesResourceWithStreamingResponse",
    "AsyncRulesResourceWithStreamingResponse",
    "MagicNetworkMonitoringResource",
    "AsyncMagicNetworkMonitoringResource",
    "MagicNetworkMonitoringResourceWithRawResponse",
    "AsyncMagicNetworkMonitoringResourceWithRawResponse",
    "MagicNetworkMonitoringResourceWithStreamingResponse",
    "AsyncMagicNetworkMonitoringResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .ca import CA as CA
from .ip_rule import IPRule as IPRule
from .email_rule import EmailRule as EmailRule
from .group_rule import GroupRule as GroupRule
from .access_rule import AccessRule as AccessRule
from .domain_rule import DomainRule as DomainRule
from .country_rule import CountryRule as CountryRule
from .ip_list_rule import IPListRule as IPListRule
from .everyone_rule import EveryoneRule as EveryoneRule
from .ip_rule_param import IPRuleParam as IPRuleParam
from .ca_list_params import CAListParams as CAListParams
from .email_list_rule import EmailListRule as EmailListRule
from .okta_group_rule import OktaGroupRule as OktaGroupRule
from .saml_group_rule import SAMLGroupRule as SAMLGroupRule
from .azure_group_rule import AzureGroupRule as AzureGroupRule
from .certificate_rule import CertificateRule as CertificateRule
from .email_rule_param import EmailRuleParam as EmailRuleParam
from .group_rule_param import GroupRuleParam as GroupRuleParam
from .access_rule_param import AccessRuleParam as AccessRuleParam
from .domain_rule_param import DomainRuleParam as DomainRuleParam
from .gsuite_group_rule import GSuiteGroupRule as GSuiteGroupRule
from .ca_delete_response import CADeleteResponse as CADeleteResponse
from .country_rule_param import CountryRuleParam as CountryRuleParam
from .ip_list_rule_param import IPListRuleParam as IPListRuleParam
from .policy_list_params import PolicyListParams as PolicyListParams
from .service_token_rule import ServiceTokenRule as ServiceTokenRule
from .everyone_rule_param import EveryoneRuleParam as EveryoneRuleParam
from .policy_get_response import PolicyGetResponse as PolicyGetResponse
from .setting_edit_params import SettingEditParams as SettingEditParams
from .policy_create_params import PolicyCreateParams as PolicyCreateParams
from .policy_list_response import PolicyListResponse as PolicyListResponse
from .policy_update_params import PolicyUpdateParams as PolicyUpdateParams
from .email_list_rule_param import EmailListRuleParam as EmailListRuleParam
from .okta_group_rule_param import OktaGroupRuleParam as OktaGroupRuleParam
from .saml_group_rule_param import SAMLGroupRuleParam as SAMLGroupRuleParam
from .setting_edit_response import SettingEditResponse as SettingEditResponse
from .setting_update_params import SettingUpdateParams as SettingUpdateParams
from .user_policy_check_geo import UserPolicyCheckGeo as UserPolicyCheckGeo
from .azure_group_rule_param import AzureGroupRuleParam as AzureGroupRuleParam
from .certificate_rule_param import CertificateRuleParam as CertificateRuleParam
from .policy_create_response import PolicyCreateResponse as PolicyCreateResponse
from .policy_delete_response import PolicyDeleteResponse as PolicyDeleteResponse
from .policy_update_response import PolicyUpdateResponse as PolicyUpdateResponse
from .gsuite_group_rule_param import GSuiteGroupRuleParam as GSuiteGroupRuleParam
from .setting_update_response import SettingUpdateResponse as SettingUpdateResponse
from .external_evaluation_rule import ExternalEvaluationRule as ExternalEvaluationRule
from .github_organization_rule import GitHubOrganizationRule as GitHubOrganizationRule
from .policy_test_get_response import PolicyTestGetResponse as PolicyTestGetResponse
from .service_token_rule_param import ServiceTokenRuleParam as ServiceTokenRuleParam
from .policy_test_create_params import PolicyTestCreateParams as PolicyTestCreateParams
from .access_device_posture_rule import AccessDevicePostureRule as AccessDevicePostureRule
from .authentication_method_rule import AuthenticationMethodRule as AuthenticationMethodRule
from .policy_test_create_response import PolicyTestCreateResponse as PolicyTestCreateResponse
from .any_valid_service_token_rule import AnyValidServiceTokenRule as AnyValidServiceTokenRule
from .external_evaluation_rule_param import ExternalEvaluationRuleParam as ExternalEvaluationRuleParam
from .github_organization_rule_param import GitHubOrganizationRuleParam as GitHubOrganizationRuleParam
from .user_policy_check_list_response import UserPolicyCheckListResponse as UserPolicyCheckListResponse
from .access_device_posture_rule_param import AccessDevicePostureRuleParam as AccessDevicePostureRuleParam
from .authentication_method_rule_param import AuthenticationMethodRuleParam as AuthenticationMethodRuleParam
from .any_valid_service_token_rule_param import AnyValidServiceTokenRuleParam as AnyValidServiceTokenRuleParam



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.magic_transit.sites import wan_edit_params, wan_create_params, wan_update_params
from ....types.magic_transit.sites.wan import WAN
from ....types.magic_transit.sites.wan_static_addressing_param import WANStaticAddressingParam

__all__ = ["WANsResource", "AsyncWANsResource"]


class WANsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> WANsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return WANsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WANsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return WANsResourceWithStreamingResponse(self)

    def create(
        self,
        site_id: str,
        *,
        account_id: str,
        physport: int,
        name: str | Omit = omit,
        priority: int | Omit = omit,
        static_addressing: WANStaticAddressingParam | Omit = omit,
        vlan_tag: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[WAN]:
        """
        Creates a new Site WAN.

        Args:
          account_id: Identifier

          site_id: Identifier

          static_addressing: (optional) if omitted, use DHCP. Submit secondary_address when site is in high
              availability mode.

          vlan_tag: VLAN ID. Use zero for untagged.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/sites/{site_id}/wans", account_id=account_id, site_id=site_id),
            page=SyncSinglePage[WAN],
            body=maybe_transform(
                {
                    "physport": physport,
                    "name": name,
                    "priority": priority,
                    "static_addressing": static_addressing,
                    "vlan_tag": vlan_tag,
                },
                wan_create_params.WANCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=WAN,
            method="post",
        )

    def update(
        self,
        wan_id: str,
        *,
        account_id: str,
        site_id: str,
        name: str | Omit = omit,
        physport: int | Omit = omit,
        priority: int | Omit = omit,
        static_addressing: WANStaticAddressingParam | Omit = omit,
        vlan_tag: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WAN:
        """
        Update a specific Site WAN.

        Args:
          account_id: Identifier

          site_id: Identifier

          wan_id: Identifier

          static_addressing: (optional) if omitted, use DHCP. Submit secondary_address when site is in high
              availability mode.

          vlan_tag: VLAN ID. Use zero for untagged.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        if not wan_id:
            raise ValueError(f"Expected a non-empty value for `wan_id` but received {wan_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/magic/sites/{site_id}/wans/{wan_id}",
                account_id=account_id,
                site_id=site_id,
                wan_id=wan_id,
            ),
            body=maybe_transform(
                {
                    "name": name,
                    "physport": physport,
                    "priority": priority,
                    "static_addressing": static_addressing,
                    "vlan_tag": vlan_tag,
                },
                wan_update_params.WANUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WAN]._unwrapper,
            ),
            cast_to=cast(Type[WAN], ResultWrapper[WAN]),
        )

    def list(
        self,
        site_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[WAN]:
        """
        Lists Site WANs associated with an account.

        Args:
          account_id: Identifier

          site_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/sites/{site_id}/wans", account_id=account_id, site_id=site_id),
            page=SyncSinglePage[WAN],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=WAN,
        )

    def delete(
        self,
        wan_id: str,
        *,
        account_id: str,
        site_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WAN:
        """
        Remove a specific Site WAN.

        Args:
          account_id: Identifier

          site_id: Identifier

          wan_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        if not wan_id:
            raise ValueError(f"Expected a non-empty value for `wan_id` but received {wan_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/magic/sites/{site_id}/wans/{wan_id}",
                account_id=account_id,
                site_id=site_id,
                wan_id=wan_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WAN]._unwrapper,
            ),
            cast_to=cast(Type[WAN], ResultWrapper[WAN]),
        )

    def edit(
        self,
        wan_id: str,
        *,
        account_id: str,
        site_id: str,
        name: str | Omit = omit,
        physport: int | Omit = omit,
        priority: int | Omit = omit,
        static_addressing: WANStaticAddressingParam | Omit = omit,
        vlan_tag: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WAN:
        """
        Patch a specific Site WAN.

        Args:
          account_id: Identifier

          site_id: Identifier

          wan_id: Identifier

          static_addressing: (optional) if omitted, use DHCP. Submit secondary_address when site is in high
              availability mode.

          vlan_tag: VLAN ID. Use zero for untagged.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        if not wan_id:
            raise ValueError(f"Expected a non-empty value for `wan_id` but received {wan_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/magic/sites/{site_id}/wans/{wan_id}",
                account_id=account_id,
                site_id=site_id,
                wan_id=wan_id,
            ),
            body=maybe_transform(
                {
                    "name": name,
                    "physport": physport,
                    "priority": priority,
                    "static_addressing": static_addressing,
                    "vlan_tag": vlan_tag,
                },
                wan_edit_params.WANEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WAN]._unwrapper,
            ),
            cast_to=cast(Type[WAN], ResultWrapper[WAN]),
        )

    def get(
        self,
        wan_id: str,
        *,
        account_id: str,
        site_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WAN:
        """
        Get a specific Site WAN.

        Args:
          account_id: Identifier

          site_id: Identifier

          wan_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        if not wan_id:
            raise ValueError(f"Expected a non-empty value for `wan_id` but received {wan_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/magic/sites/{site_id}/wans/{wan_id}",
                account_id=account_id,
                site_id=site_id,
                wan_id=wan_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WAN]._unwrapper,
            ),
            cast_to=cast(Type[WAN], ResultWrapper[WAN]),
        )


class AsyncWANsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncWANsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWANsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWANsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncWANsResourceWithStreamingResponse(self)

    def create(
        self,
        site_id: str,
        *,
        account_id: str,
        physport: int,
        name: str | Omit = omit,
        priority: int | Omit = omit,
        static_addressing: WANStaticAddressingParam | Omit = omit,
        vlan_tag: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[WAN, AsyncSinglePage[WAN]]:
        """
        Creates a new Site WAN.

        Args:
          account_id: Identifier

          site_id: Identifier

          static_addressing: (optional) if omitted, use DHCP. Submit secondary_address when site is in high
              availability mode.

          vlan_tag: VLAN ID. Use zero for untagged.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/sites/{site_id}/wans", account_id=account_id, site_id=site_id),
            page=AsyncSinglePage[WAN],
            body=maybe_transform(
                {
                    "physport": physport,
                    "name": name,
                    "priority": priority,
                    "static_addressing": static_addressing,
                    "vlan_tag": vlan_tag,
                },
                wan_create_params.WANCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=WAN,
            method="post",
        )

    async def update(
        self,
        wan_id: str,
        *,
        account_id: str,
        site_id: str,
        name: str | Omit = omit,
        physport: int | Omit = omit,
        priority: int | Omit = omit,
        static_addressing: WANStaticAddressingParam | Omit = omit,
        vlan_tag: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WAN:
        """
        Update a specific Site WAN.

        Args:
          account_id: Identifier

          site_id: Identifier

          wan_id: Identifier

          static_addressing: (optional) if omitted, use DHCP. Submit secondary_address when site is in high
              availability mode.

          vlan_tag: VLAN ID. Use zero for untagged.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        if not wan_id:
            raise ValueError(f"Expected a non-empty value for `wan_id` but received {wan_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/magic/sites/{site_id}/wans/{wan_id}",
                account_id=account_id,
                site_id=site_id,
                wan_id=wan_id,
            ),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "physport": physport,
                    "priority": priority,
                    "static_addressing": static_addressing,
                    "vlan_tag": vlan_tag,
                },
                wan_update_params.WANUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WAN]._unwrapper,
            ),
            cast_to=cast(Type[WAN], ResultWrapper[WAN]),
        )

    def list(
        self,
        site_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[WAN, AsyncSinglePage[WAN]]:
        """
        Lists Site WANs associated with an account.

        Args:
          account_id: Identifier

          site_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/sites/{site_id}/wans", account_id=account_id, site_id=site_id),
            page=AsyncSinglePage[WAN],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=WAN,
        )

    async def delete(
        self,
        wan_id: str,
        *,
        account_id: str,
        site_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WAN:
        """
        Remove a specific Site WAN.

        Args:
          account_id: Identifier

          site_id: Identifier

          wan_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        if not wan_id:
            raise ValueError(f"Expected a non-empty value for `wan_id` but received {wan_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/magic/sites/{site_id}/wans/{wan_id}",
                account_id=account_id,
                site_id=site_id,
                wan_id=wan_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WAN]._unwrapper,
            ),
            cast_to=cast(Type[WAN], ResultWrapper[WAN]),
        )

    async def edit(
        self,
        wan_id: str,
        *,
        account_id: str,
        site_id: str,
        name: str | Omit = omit,
        physport: int | Omit = omit,
        priority: int | Omit = omit,
        static_addressing: WANStaticAddressingParam | Omit = omit,
        vlan_tag: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WAN:
        """
        Patch a specific Site WAN.

        Args:
          account_id: Identifier

          site_id: Identifier

          wan_id: Identifier

          static_addressing: (optional) if omitted, use DHCP. Submit secondary_address when site is in high
              availability mode.

          vlan_tag: VLAN ID. Use zero for untagged.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        if not wan_id:
            raise ValueError(f"Expected a non-empty value for `wan_id` but received {wan_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/magic/sites/{site_id}/wans/{wan_id}",
                account_id=account_id,
                site_id=site_id,
                wan_id=wan_id,
            ),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "physport": physport,
                    "priority": priority,
                    "static_addressing": static_addressing,
                    "vlan_tag": vlan_tag,
                },
                wan_edit_params.WANEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WAN]._unwrapper,
            ),
            cast_to=cast(Type[WAN], ResultWrapper[WAN]),
        )

    async def get(
        self,
        wan_id: str,
        *,
        account_id: str,
        site_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WAN:
        """
        Get a specific Site WAN.

        Args:
          account_id: Identifier

          site_id: Identifier

          wan_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        if not wan_id:
            raise ValueError(f"Expected a non-empty value for `wan_id` but received {wan_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/magic/sites/{site_id}/wans/{wan_id}",
                account_id=account_id,
                site_id=site_id,
                wan_id=wan_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WAN]._unwrapper,
            ),
            cast_to=cast(Type[WAN], ResultWrapper[WAN]),
        )


class WANsResourceWithRawResponse:
    def __init__(self, wans: WANsResource) -> None:
        self._wans = wans

        self.create = to_raw_response_wrapper(
            wans.create,
        )
        self.update = to_raw_response_wrapper(
            wans.update,
        )
        self.list = to_raw_response_wrapper(
            wans.list,
        )
        self.delete = to_raw_response_wrapper(
            wans.delete,
        )
        self.edit = to_raw_response_wrapper(
            wans.edit,
        )
        self.get = to_raw_response_wrapper(
            wans.get,
        )


class AsyncWANsResourceWithRawResponse:
    def __init__(self, wans: AsyncWANsResource) -> None:
        self._wans = wans

        self.create = async_to_raw_response_wrapper(
            wans.create,
        )
        self.update = async_to_raw_response_wrapper(
            wans.update,
        )
        self.list = async_to_raw_response_wrapper(
            wans.list,
        )
        self.delete = async_to_raw_response_wrapper(
            wans.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            wans.edit,
        )
        self.get = async_to_raw_response_wrapper(
            wans.get,
        )


class WANsResourceWithStreamingResponse:
    def __init__(self, wans: WANsResource) -> None:
        self._wans = wans

        self.create = to_streamed_response_wrapper(
            wans.create,
        )
        self.update = to_streamed_response_wrapper(
            wans.update,
        )
        self.list = to_streamed_response_wrapper(
            wans.list,
        )
        self.delete = to_streamed_response_wrapper(
            wans.delete,
        )
        self.edit = to_streamed_response_wrapper(
            wans.edit,
        )
        self.get = to_streamed_response_wrapper(
            wans.get,
        )


class AsyncWANsResourceWithStreamingResponse:
    def __init__(self, wans: AsyncWANsResource) -> None:
        self._wans = wans

        self.create = async_to_streamed_response_wrapper(
            wans.create,
        )
        self.update = async_to_streamed_response_wrapper(
            wans.update,
        )
        self.list = async_to_streamed_response_wrapper(
            wans.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            wans.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            wans.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            wans.get,
        )
